use utf8;
use strict;
use warnings FATAL => 'all';

package Xyzzy::Generator;

use Spiffy -Base;

use DBI;
use XML::LibXML;

use Xyzzy::Document;

field 'path';
field 'fullpath';
field 'doc';
field 'dbh';
field 'req';
field 'cfg';
field 'msie';

sub db {
	my $cfg = $self->cfg;
	my $db = $self->dbh;
	return $db if defined $db;
	$db = DBI->connect_cached(
			$cfg->db_datasource,
			$cfg->db_user,
			$cfg->db_pass,
			$cfg->db_options
		);
	$db->begin_work;
	$self->dbh($db);
	return $db;
}

sub db_commit {
	my $db = $self->dbh;
	return unless defined $db;
	$db->commit;
#	$db->disconnect;
	$self->dbh(undef);
}

sub db_rollback {
	my $db = $self->dbh;
	return unless defined $db;
	$db->rollback;
#	$db->disconnect;
	$self->dbh(undef);
}

sub find_template {
	my $template = shift;
	my $dir = $self->h->cfg->templatedir;
	return "$dir/$template.xml"
}

sub addTemplates {
	my $doc = $self->doc;
	my $dir = $self->cfg->templatedir;
	$doc->addTemplates(map { "$dir/$_.xml" } @_);
}

sub addContent {
	my $doc = $self->doc;
	my $dir = $self->cfg->staticrootdir;
	$doc->addContent(map { "$dir/$_.xml" } @_);
}

sub init {
	my $doc = Xyzzy::Document->xhtml;
	$self->doc($doc)
}

sub setup {
	my $doc = $self->doc;
	$self->addTemplates('base');
}

sub build {
	my $doc = $self->doc;

	my $title = $doc->getElementById('title');
	$title->appendTextNode(ref $self)
		if defined $title;

	my $headline = $doc->getElementById('headline');
	$headline->appendTextNode(ref $self)
		if defined $headline;
}

sub finalize {
	my $doc = $self->doc;
	my $req = $self->req;

	$doc->xhtml_cleanup;
#	$doc->validate;

	if($self->msie) {
		$req->mimetype('text/html', charset => 'UTF-8');
		$req->content($doc->toStringHTML);
	} else {
		$req->mimetype('application/xhtml+xml');
		$req->content($doc->toString);
	}
}

sub new {
	my ($req, $config, $path) = @_;

	my $ua = $req->user_agent;
	my $msie = defined $ua && index($ua, 'MSIE') != -1;

	return super(path => $path, cfg => $config, req => $req, msie => $msie);
}

sub xyzzy {
	$self->init;
	$self->setup;
	$self->build;
	$self->finalize;
}

sub work {
	$self->xyzzy;
}

sub generate {
	my $gen = $self->new(@_);

	eval { $gen->work };
	my $err = $@;
	if($err) {
		eval { $gen->db_rollback };
		warn "db_rollback: $@" if $@;
		die $err;
	}

	$gen->db_commit;

	return $gen;
}
